<?php

namespace Drp\ProjectHelpers\Tests\Concerns;

use Illuminate\Http\UploadedFile;
use League\Flysystem\Filesystem;
use League\Flysystem\Vfs\VfsAdapter;
use Mockery;
use Symfony\Component\DomCrawler\Form;
use VirtualFileSystem\FileSystem as Vfs;

/**
 * Class InteractsWithFilesystem
 *
 * @package Drp\ProjectHelpers
 */
trait InteractsWithFilesystem
{

    /**
     * @var
     */
    protected $filesystem;

    /**
     * @param array $structure
     * @return \League\Flysystem\Filesystem
     */
    protected function setVirtualFileStructure(array $structure = [])
    {
        $vfs = new Vfs;
        $vfs->createStructure($structure);

        $adapter = new VfsAdapter($vfs);
        $filesystem = new Filesystem($adapter);

        app('filesystem')->extend('vfs', function () use ($filesystem) {
            return $filesystem;
        });

        config([
            'filesystems.disks.virtual.driver' => 'vfs',
            'filesystems.default' => 'virtual',
        ]);

        $this->filesystem = $filesystem;
        return $filesystem;
    }

    /**
     * @param $path
     * @return $this
     */
    protected function assertFileInFileSystem($path)
    {
        $this->assertTrue($this->filesystem->has($path), 'Cannot find file in directory');
        return $this;
    }

    /**
     * @param $file
     * @param $hashName
     * @return Mockery\MockInterface
     */
    protected function createFileUploadMock($file, $hashName)
    {
        $file = Mockery::mock(UploadedFile::class . '[hashName]', [$file, $file, null, null, null, true]);
        $file->shouldReceive('hashName')
            ->atMost(1)
            ->andReturn($hashName);

        return $file;
    }

    /**
     * Convert the given uploads to UploadedFile instances.
     *
     * @param  \Symfony\Component\DomCrawler\Form $form
     * @param  array $uploads
     * @return array
     */
    protected function convertUploadsForTesting(Form $form, array $uploads)
    {
        $files = $form->getFiles();
        $names = array_keys($files);

        $files = array_map(function (array $file, $name) use ($uploads) {
            return isset($uploads[$name]) ? $uploads[$name] : null;
        }, $files, $names);


        $arr = [];
        collect(array_combine($names, $files))
            ->each(function ($file, $key) use (&$arr) {
                preg_match('/([a-z_]*)([[]\d[]])/', $key, $matches);
                if (count($matches) > 0) {
                    $key = $matches[1];
                    $arr[$key][] = $file;
                } else {
                    $arr[$key] = $file;
                }
            });

        return $arr;
    }
}

